import plants, tilemap, basket, utils, items
from pubsub import Publisher

class LevelList(object):
    def __init__(self, window):
        self.Window = window
        self.Levels = [TomatoLevel, CarrotLevel]
        self.CurrentLevel = -1
        self.Level = None
        
    def Next(self):
        self.CurrentLevel += 1
        if self.CurrentLevel < len(self.Levels):
            self.Level = self.Levels[self.CurrentLevel](self.Window)
            self.Level.Number = self.CurrentLevel + 1 # Use 1-based here.
            Publisher.sendMessage("level.started", self.Level)
        else:
            Publisher.sendMessage("level.nomore")
    
    def tick(self, dt):
        self.Level.tick(dt)
        
    def draw(self, parent):
        self.Level.draw(parent)

class BaseLevel(utils.Subscribable):
    def __init__(self, window, name, duration, map=None):
        self.Window = window
        self.Name = name
        self.Duration = self.TimeLeft = duration
        self.Complete = False
        self.Map = map
        self.Basket = basket.Basket()
        self.BasketGoals = basket.Basket()
        self.Resources, self.Plants, self.Animals = (items.Hand,), (), ()
        self.Instruction = None
        
        self.Subscriptions = (
            (self.onItemPicked, "item.picked"),
        )
        self.subscribe()
        
    def setInstruction(self, instruction):
        if self.Instruction:
            self.Instruction.stop()
        self.Instruction = instruction
        if self.Instruction: # Can pass None.
            self.Instruction.start()
        
    def clear(self):
        self.unsubscribe()
        self.setInstruction(None)
        self.Map.clear()
        
    def onItemPicked(self, message):
        item = message.data
        self.Basket.append(item)
        
    def isWon(self):
        return self.BasketGoals.isSatisfiedBy(self.Basket)
    
    def retry(self):
        self.TimeLeft = self.Duration
        self.Map.unhighlight()
        self.Complete = False
        
    def tick(self, dt):
        self.Map.tick(dt)
        if not self.Complete:
            if self.TimeLeft == 0:
                Publisher.sendMessage("level.ended", self)
                self.Complete = True
            else:
                self.TimeLeft = utils.clamp(self.TimeLeft-dt, 0)
            
        if self.Instruction:
            self.Instruction.tick(dt)
    
    def draw(self, parent):
        self.Map.draw(parent)
        
class TomatoLevel(BaseLevel):
    def __init__(self, window):
        BaseLevel.__init__(self, window, "Tomatoes!", 50)
        self.firstTomatoDropped = False
        self.firstTomatoPicked = False
        self.Map = tilemap.Map(window)
        self.Plants = (plants.Tomato,)
        
        self.BasketGoals = basket.Basket({
            plants.Tomato: 2,
        })
        
        self.Instructions = (
            utils.Instruction("Welcome! Each level, your goal is to fill the basket in the bottom right by harvesting produce."),
            utils.Instruction("Try planting a tomato by pressing the 'T' key, and clicking in a square."),
            utils.Instruction("Plants fade in as they grow, then rotate upright as they produce goods."),
            utils.Instruction("Once a plant is upright, press the 'H' key and click on it to Harvest it. " +
                              "It will then be added to your produce basket, and start producing again."),
            utils.Instruction("Great job! That's the basic idea. Now just harvest it again to meet the goal."),
        )
        self.setInstruction(self.Instructions[0])
        Publisher.subscribe(self.onInstructionComplete, "instruction.complete")
        Publisher.subscribe(self.onItemDropped, "item.dropped")
        Publisher.subscribe(self.onItemPicked2, "item.picked")
        
    def onItemPicked2(self, message):
        if not self.firstTomatoPicked:
            self.setInstruction(self.Instructions[4])
            self.firstTomatoPicked = True
        
    def onItemDropped(self, message):
        item = message.data
        if self.firstTomatoDropped == False and item == plants.Tomato:
            self.setInstruction(self.Instructions[2])
            self.firstTomatoDropped = True
        
    def onInstructionComplete(self, message):
        instr = message.data
        if instr is self.Instructions[0]:
            self.setInstruction(self.Instructions[1])
        elif instr is self.Instructions[2]:
            self.setInstruction(self.Instructions[3])
            
    
class CarrotLevel(BaseLevel):
    def __init__(self, window):
        BaseLevel.__init__(self, window, "Carrots!", 60)
        self.Map = tilemap.Map(window)
        self.Plants = (plants.Carrot,)
    
        self.BasketGoals = basket.Basket({
            plants.Carrot: 10,
        })
        
        self.Instructions = (
            utils.Instruction("To make your goals this level, you'll need more than one plant."),
        )
        self.setInstruction(self.Instructions[0])
