import pyglet, utils
from pubsub import Publisher

menuBatch = pyglet.graphics.Batch()
X = 600

class Menu(utils.Subscribable):
    def __init__(self, parent):
        self.parent = parent
        self.Level = None
        self.CurrentSelection = None
        
        self.menuBack = pyglet.resource.image("menuback.png")
        
        self.resourcesMenu = ResourcesMenu(self, X+3, self.parent.height-50)
        self.plantsMenu = PlantsMenu(self, X+3, self.parent.height-180)
        self.animalsMenu = AnimalsMenu(self, X+3, self.parent.height-330)
        self.basketMenu = BasketMenu(self, X+3, self.parent.height-480)
        self.subMenus = (self.resourcesMenu, self.plantsMenu, self.animalsMenu, self.basketMenu)
        
        self.headerBase = "Level %i: %s"
        self.headerLabel = pyglet.text.Label(x=X+3, font_size=14, y=parent.height-20, batch=menuBatch)
        self.footerBase = "Time Left: %.1f"
        self.footerLabel = pyglet.text.Label(font_size=16, x=X+3, y=5, batch=menuBatch)
        
        self.Subscriptions = (
            (self.onLevelStarted, "level.started"),
            (self.onKeyPress, "keypress"),
            (self.onLeftClick, "click.left"),
            (self.onItemDropped, "item.dropped"),
        )
        self.subscribe()
        
    def onItemDropped(self, message):
        self.resetCursor()
        
    def resetCursor(self):
        self.parent.set_mouse_cursor()
        self.CurrentSelection = None
        
    def setCursor(self, item):
        # Set the mouse cursor to the match's icon.
        img = pyglet.window.ImageMouseCursor(item.IMAGE)
        img.hot_y = img.texture.height
        self.parent.set_mouse_cursor(img)
        self.CurrentSelection = item
        
    def onLeftClick(self, message):
        if self.CurrentSelection:
            Publisher.sendMessage("item.dropping", self.CurrentSelection)
        
    def onKeyPress(self, message):
        key = int(message.topic[-1])
        for submenu in self.subMenus:
            match = submenu.getKeyMatch(key)
            if match:
                self.setCursor(match)
                break
        
    def onLevelStarted(self, message):
        self.resetCursor()
        self.Level = message.data
        self.headerLabel.text = self.headerBase % (self.Level.Number, self.Level.Name)
        self.setupMenus()
        
    def setupMenus(self):
        for menu in self.subMenus:
            menu.setup(self.Level)
        
    def tick(self, dt):
        if self.Level:
            self.footerLabel.text = self.footerBase % self.Level.TimeLeft
        
    def draw(self):
        self.menuBack.blit(X, 0)
        #for menu in self.subMenus: menu.draw()
        menuBatch.draw()
        
        
class SubMenu(object):
    def __init__(self, parent, title, x, y):
        self.parent = parent
        self.title = title
        self.x, self.y = x, y
        self.header = pyglet.text.Label("%s:" % title, color=(0,0,0,255), font_size=16, batch=menuBatch, x=x, y=y)
        self.Items, self.Labels = [], []
        
    def clear(self):
        for label in self.Labels:
            label.delete()
        self.Items, self.Labels = [], []
        
    def setup(self, level):
        self.clear()
        
        things = getattr(level, self.title)
        for i, thing in enumerate(things):
            text = "%s (%s)" % (thing.NAME, thing.SHORTCUT)
            label = pyglet.text.Label(text, font_size=12, color=(0,0,0,255), x=self.x, y=self.y-20-i*20, batch=menuBatch)
            self.Items.append(thing)
            self.Labels.append(label)
            
    def getKeyMatch(self, key):
        match = None
        for item, label in zip(self.Items, self.Labels):
            if key == getattr(pyglet.window.key, item.SHORTCUT):
                return item
        
class ResourcesMenu(SubMenu):
    def __init__(self, parent, *args):
        SubMenu.__init__(self, parent, "Resources", *args)
        
class PlantsMenu(SubMenu):
    def __init__(self, parent, *args):
        SubMenu.__init__(self, parent, "Plants", *args)
        
class AnimalsMenu(SubMenu):
    def __init__(self, parent, *args):
        SubMenu.__init__(self, parent, "Animals", *args)
        
class BasketMenu(SubMenu):
    def __init__(self, parent, *args):
        SubMenu.__init__(self, parent, "Basket", *args)
        self.LabelBases = {}
        self.Labels = {}
        
        Publisher.subscribe(self.update, "basket.updated")
        
    def clear(self):
        for label in self.Labels.values():
            label.delete()
        self.Items, self.Labels = [], {}
        
    def setup(self, level):
        self.clear()
        for i, (goal, count) in enumerate(level.BasketGoals.items()):
            labelBase = goal.NAME + ": %i/" + str(count)
            self.LabelBases[goal] = labelBase
            self.Labels[goal] = pyglet.text.Label(font_size=12, bold=True, color=(255,0,0,200), x=self.x, y=self.y-20-i*20, batch=menuBatch)
        self.update()
            
    def update(self, message=None):
        level = self.parent.Level
        for key in level.BasketGoals:
            count = level.Basket[key]
            self.Labels[key].text = self.LabelBases[key] % count
            if count >= level.BasketGoals[key]:
                self.Labels[key].color = (50,200,50,200)
        
    def getKeyMatch(self, key):
        # There are no keybindings for this menu.
        return None
        